#import "AppWindowController.h"
#import "PreferencesWindowController.h"
#import "ImageView.h"
#import "ImageBrowserController.h"
#import "ImageBrowserItem.h"
#import "Utility.h"


@implementation AppWindowController

@synthesize window = mWindow;

-(void)awakeFromNib
{
	// initialize the image view with nil (if we don't do that, we will see
	// artefacts in the IKImageView
	[mImageView setImageWithURL:nil];

	// initialize the arrays for copy / cut / paste
	mCutFiles = [[NSMutableArray alloc] init];
	mCopiedFiles = [[NSMutableArray alloc] init];
}

-(void)dealloc
{
	[mCutFiles release];
	[mCopiedFiles release];
	[super dealloc];
}

-(void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	// configure the shared user defaults to save changes immediately
	NSUserDefaultsController * defaults = [NSUserDefaultsController sharedUserDefaultsController];
	[defaults setAppliesImmediately:YES];
}

-(BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender
{
	return YES;
}

-(void)applicationWillFinishLaunching:(NSNotification *)notification
{
	// initialize the default parameters for the application
	// first step : create objects for complex parameters
	NSData * colorData = [NSKeyedArchiver archivedDataWithRootObject:[NSColor blackColor]];
	
	// second step : create the dictionary containing the parameters keys and values
	NSMutableDictionary * defaults = [NSMutableDictionary dictionary];
	[defaults setValue:colorData forKey:@"backgroundColor"];
	[defaults setValue:[NSNumber numberWithFloat:0.38f] forKey:@"thumbnailSize"];
	[defaults setValue:[NSNumber numberWithBool:NO] forKey:@"permanentlyDelete"];
	
	// last step : register the default parameters
	[[NSUserDefaults standardUserDefaults] registerDefaults:defaults];
}

-(void)applicationWillTerminate:(NSNotification *)aNotification
{
	// restore to windowed mode before closing. If we don't do that,
	// we will save the states of the split views, and the next time the
	// application will be executed, the image view will be invisible
	if ([mImageView isInFullScreenMode] == YES)
		[mImageView toggleFullscreen];
}

-(IBAction)showPreferencesWindow:(id)sender
{
	// launch the preference window
	[[PreferencesWindowController sharedInstance] runModal];
}

-(IBAction)deleteFiles:(id)sender
{
	BOOL permanentlyDelete = [[[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"permanentlyDelete"] boolValue];
	NSArray * files = [mImageBrowserController selectedImages];
	if ([mImageView isInFullScreenMode] == YES)
	{
		// if we are in fullscreen mode, remove only the first selected image
		[Utility removeFileAtPath:[[files objectAtIndex:0] path] permanently:permanentlyDelete];
		int image = [mImageBrowserController removeFirstSelectedImage];
		
		// update the current image
		int numImages = [mImageBrowserController numberOfItemsInImageBrowser:(IKImageBrowserView *)mImageBrowser];
		if (numImages == 0)			[mImageView toggleFullscreen];
		else if (image < numImages)	[mImageBrowser setSelectionIndexes:[NSIndexSet indexSetWithIndex:image] byExtendingSelection:NO];
		else						[mImageBrowser setSelectionIndexes:[NSIndexSet indexSetWithIndex:image - 1] byExtendingSelection:NO];
	}
	else
	{
		// if not in fullscreen mode, remove all the selected images from the disk
		for (ImageBrowserItem * item in files)
			[Utility removeFileAtPath:[item path] permanently:permanentlyDelete];

		// and update the image browser
		[mImageBrowserController removeSelectedImages];
	}
}

-(IBAction)copyFiles:(id)sender
{
	[mCutFiles removeAllObjects];
	[mCopiedFiles removeAllObjects];
	for (id image in [mImageBrowserController selectedImages])
		[mCopiedFiles addObject:[[image path] copy]];
}

-(IBAction)cutFiles:(id)sender
{
	[mCutFiles removeAllObjects];
	[mCopiedFiles removeAllObjects];
	for (id image in [mImageBrowserController selectedImages])
		[mCutFiles addObject:[[image path] copy]];
}

-(IBAction)pasteFiles:(id)sender
{
	NSFileManager * fileManager = [NSFileManager defaultManager];
	if ([mCutFiles count] > 0)
	{
		// get the destination path
		NSString * destPath = [mImageBrowserController path];
		
		// only past files if the destination path is different from the source one
		if ([[[mCutFiles objectAtIndex:0] stringByDeletingLastPathComponent] isEqualToString:destPath] == NO)
		{
			for (NSString * srcFile in mCutFiles)
			{
				NSError * error;
				NSString * destFile = [destPath stringByAppendingPathComponent:[srcFile lastPathComponent]];
				if ([fileManager moveItemAtPath:srcFile toPath:destFile error:&error] == NO)
				{
					NSLog(@"error : %@", [error description]);
					NSLog(@"source file : %@", srcFile);
					NSLog(@"dest file : %@", destFile);
				}
				else
					[mImageBrowserController addImage:destFile];
			}
		}
	}
	else if ([mCopiedFiles count] > 0)
	{
		// get the destination path
		NSString * destPath = [mImageBrowserController path];
		
		// only paste files if the destination path is different from the source one
		if ([[[mCopiedFiles objectAtIndex:0] stringByDeletingLastPathComponent] isEqualToString:destPath] == NO)
		{
			for (NSString * srcFile in mCopiedFiles)
			{
				NSError * error;
				NSString * destFile = [destPath stringByAppendingPathComponent:[srcFile lastPathComponent]];
				if ([fileManager copyItemAtPath:srcFile toPath:destFile error:&error] == NO)
				{
					NSLog(@"error : %@", [error description]);
					NSLog(@"source file : %@", srcFile);
					NSLog(@"dest file : %@", destFile);
				}
				else
					[mImageBrowserController addImage:destFile];
			}
		}
	}
	[mCutFiles removeAllObjects];
	[mCopiedFiles removeAllObjects];
}

@end
